#ifndef AUDIOPIPELINE_AUDIO_FILTER_PARAM_EQ_H__INCLUDED
#define AUDIOPIPELINE_AUDIO_FILTER_PARAM_EQ_H__INCLUDED

#include "audio.h"

typedef enum _EQBandType {
    EQTypeLowPass,      /** simple low pass filter */
    EQTypeLowShelf,     /** low shelf filter, basically all values before the center frequency get modified by the gain*/
    EQTypeBandPass,     /** only frequencies around the center frequency will pass through */
    EQTypePeak,         /** amplify frequencies around the center frequency */
    EQTypeNotch,        /** duck frequencies around the center frequency */
    EQTypeHighPass,     /** simple high pass filter */
    EQTypeHighShelf     /** high shelf, like the low shelf filter but affects all frequencies after the center */
} EQBandType;

typedef struct _EQBand {
    EQBandType type;    /** type of filter for this band */
    float frequency;    /** EQ band center frequency */
    float Q;            /** Filter bandwidth, not used by shelf filters */
    float gain;         /** Only used for Peak, Notch and Shelf filters*/
} EQBand;

/**
 * Creates a filter element which applies a parametric EQ to the audio samples
 *
 * @param bands Array of EQ bands to apply, start with low shelf and end on high shelf if possible
 * @param numBands number of bands in the array
 * @param prescale prescale all bands by this value
 *
 * @returns Initialized `AudioPipelineElement` that can be used in call to `audio_pipeline_assemble`
 */
AudioPipelineElement *audio_filter_param_eq(EQBand *bands, int numBands, float prescale);

#undef SECOND_ORDER_FILTER   /* use a second biquad stage for steeper cutoff frequencies, needs more processing power */
#undef FIXEDPOINT            /* run in fixed point mode, floats are used only on initialization, filter unstable below 300 Hz! */
#define LOW_PRECISION        /* run filter on float32 instead of double precision */

#endif /* AUDIOPIPELINE_AUDIO_FILTER_PARAM_EQ_H__INCLUDED */

